// @ts-check
import { randomBytes } from "crypto";
import { hashToken } from "../utils";
/**
 * @typedef {import("types/providers").EmailConfig} EmailConfig
 */
/**
 * Starts an e-mail login flow, by generating a token,
 * and sending it to the user's e-mail (with the help of a DB adapter)
 * @param {string} identifier
 * @param {import("types/internals").InternalOptions<EmailConfig>} options
 */
export default async function email(identifier, options) {
    var _a, _b, _c;
    const { baseUrl, basePath, adapter, provider, logger } = options;
    // Generate token
    const token = (_b = (await ((_a = provider.generateVerificationToken) === null || _a === void 0 ? void 0 : _a.call(provider)))) !== null && _b !== void 0 ? _b : randomBytes(32).toString("hex");
    const ONE_DAY_IN_SECONDS = 86400;
    const expires = new Date(Date.now() + ((_c = provider.maxAge) !== null && _c !== void 0 ? _c : ONE_DAY_IN_SECONDS) * 1000);
    // Save in database
    // @ts-expect-error
    await adapter.createVerificationToken({
        identifier,
        token: hashToken(token, options),
        expires,
    });
    // Generate a link with email and the unhashed token
    const params = new URLSearchParams({ token, email: identifier });
    const url = `${baseUrl}${basePath}/callback/${provider.id}?${params}`;
    try {
        // Send to user
        await provider.sendVerificationRequest({
            identifier,
            token,
            expires,
            url,
            provider,
        });
    }
    catch (error) {
        logger.error("SEND_VERIFICATION_EMAIL_ERROR", {
            identifier,
            url,
            error,
        });
        throw new Error("SEND_VERIFICATION_EMAIL_ERROR");
    }
}
